program MIDIRec;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Windows,
  ShellAPI,
  Exec,
  vtsSoundUtils in 'vtsSoundUtils.pas',
  FileCollector in 'FileCollector.pas';

{$INCLUDE Config.inc}

{$R *.res}

var
  Terminated: boolean;
  BreakDesired: boolean;
  CanHalt: boolean;

function ConsoleEventProc(CtrlType: DWORD): Bool; stdcall; far;
resourcestring
  LNG_USER_TERM = 'User terminated program.';
  LNG_USER_BREAK = 'User breaked recording.';
begin
  result := false;

  if CanHalt then
  begin
    Halt;
    Exit;
  end;

  if Terminated or BreakDesired then
  begin
    // Weiterhin ignorieren
    result := true;
    exit;
  end;

  case CtrlType of
    CTRL_C_EVENT, CTRL_CLOSE_EVENT, CTRL_LOGOFF_EVENT, CTRL_SHUTDOWN_EVENT:
    begin
      WriteLn(LNG_USER_TERM);
      Terminated := true;
      if CanHalt then Halt;
      result := true; // Ignorieren
    end;
    CTRL_BREAK_EVENT:
    begin
      WriteLn(LNG_USER_BREAK);
      BreakDesired := true;
      result := true; // Ignorieren
    end;
  end;
end;

function InterruptCallback(): boolean;
begin
  result := not (Terminated or BreakDesired or CanHalt);
  // BreakDesired := false;
end;

procedure PrepareExit();
var
  dummy: string;
resourcestring
  LNG_PRESS_ENTER = 'Press ENTER to exit...';
begin
  StopRecording(); // For security
  CanHalt := true; // Break's ReadLn()
  WriteLn('');
  WriteLn(LNG_PRESS_ENTER);
  ReadLn(dummy);
end;

const
  EX_OK = 0;

{$IFDEF USE_HARDLIM}
function HardLimit(filename: string): boolean;
resourcestring
  LNG_HARDLIM = 'Hard limiting ...';
const
  tmpfn = '~HardLimTmp.wav'; // FUTURE: In ein Temp-Verzeichnis des Systems kopieren
  EXE_SOX = 'sox\sox.exe';
  VerboseLevel = 2;
  ParamProto = '-V%d --norm "%s" --bits %d --channels %d "%s" rate %d dither -s -S';
var
  errc: DWord;
begin
  result := false;
  if Terminated or BreakDesired then exit;
  DeleteFile(PChar(tmpfn));
  if FileExists(tmpfn) then exit;
  WriteLn(LNG_HARDLIM);
  ShellExecAndWait(EXE_SOX,
    Format(ParamProto, [VerboseLevel, filename, BPS, CH, tmpfn, SPS]),
    ExtractFilePath(ParamStr(0)), SW_MINIMIZE, errc);
  if errc <> EX_OK then
  begin
    DeleteFile(PChar(tmpfn));
    exit;
  end;
  result := FileExists(tmpfn);
  if result then
  begin
    DeleteFile(PChar(filename));
    if FileExists(filename) then
    begin
      result := false;
      exit;
    end;
    result := Movefile(PChar(tmpfn), PChar(filename));
  end;
end;
{$ENDIF} // IFDEF USE_HARDLIM

{$IFDEF USE_LAME}
function MP3Conv(filename: string): boolean;
resourcestring
  LNG_LAME = 'Converting with LAME ...';
const
  EXE_LAME = 'lame\lame.exe';
  MP3_EXT = '.mp3';
var
  outfn: string;
  errc: DWord;
begin
  result := false;
  if Terminated or BreakDesired then exit;

  WriteLn(LNG_LAME);

  outfn := ChangeFileExt(filename, MP3_EXT);

  DeleteFile(PChar(outfn));
  if FileExists(outfn) then exit;

  // WARNUNG: LAME IST NICHT WINDOWS 95 COMPATIBLE!
  ShellExecAndWait(EXE_LAME,
    '"'+filename+'" "'+outfn+'"',
    ExtractFilePath(ParamStr(0)), SW_MINIMIZE, errc);
  if errc <> EX_OK then
  begin
    DeleteFile(PChar(outfn));
    exit;
  end;

  result := FileExists(outfn);
end;
{$ENDIF} // IFDEF USE_LAME

var
  i: integer;
  filename, filename2: string;
  fcol: TParamFilesCollector;

resourcestring
  LNG_CAPTION = 'VIATHINKSOFT BATCH MIDI RECORDER!'+#13#10+
    '(C) 2010 ViaThinkSoft. All rights reserved.'+#13#10+
    #13#10+
    'Version 1.2 [2010-04-06]';
  LNG_SYNTAX = 'Syntax: MIDIRec filename1 [filename2 ...]';
  LNG_INFO = 'Note: If the resulting files are silent, please check if'+#13#10+
    '"MIDI Out" (or similar) is defined as your default recording device!';
  LNG_MCI_ERROR = 'MCI ERROR: %s';
  LNG_FINISHED = 'Finished';
  LNG_FILE = 'File: %s';
  LNG_RECORDING = 'Recording ...';
  LNG_SUCCEED = '=> Succeed';
  LNG_FAILED = '=> Failed';
  LNG_CANNOT_HANDLE_FOLDER = 'Cannot handle folder: "%s"!';
const
  WAV_EXT = '.wav';
begin
  WriteLn(LNG_CAPTION);
  WriteLn('');

  if ParamCount() < 1 then
  begin
    WriteLn(LNG_SYNTAX);
    PrepareExit();
    exit;
  end;

  WriteLn(LNG_INFO);

  SetConsoleCtrlHandler(@ConsoleEventProc, true);
  MIDIPlayingCallback := @InterruptCallback;

  fcol := TParamFilesCollector.Create(emNormal);
  try
    fcol.AddFromParamString(1);

    for i := 0 to fcol.CollectedFoldersCount - 1 do
    begin
      WriteLn(Format(LNG_CANNOT_HANDLE_FOLDER, [fcol.CollectedFolders[i]]));
    end;

    for i := 0 to fcol.CollectedFilesCount - 1 do
    begin
      filename := fcol.CollectedFiles[i];
      filename2 := ChangeFileExt(filename, WAV_EXT);

      WriteLn('');
      WriteLn(Format(LNG_FILE, [filename]));
      WriteLn(LNG_RECORDING);

      if not StartRecording() then
      begin
        WriteLn(Format(LNG_MCI_ERROR, [getLastMCIError()]));
        StopRecording();
        if Terminated then
        begin
          PrepareExit();
          exit;
        end else Continue;
      end;

      if not playMIDIAndWait(filename) then
      begin
        WriteLn(Format(LNG_MCI_ERROR, [getLastMCIError()]));
        StopRecording();
        if Terminated then
        begin
          PrepareExit();
          exit;
        end else Continue;
      end;

      if CanHalt then Halt; // hier nicht bentigt

      if BreakDesired then
      begin
        StopRecording();
        BreakDesired := false;
      end
      else
      begin
        if not StopRecording(filename2) then
        begin
          WriteLn(Format(LNG_MCI_ERROR, [getLastMCIError()]));
          if Terminated then
          begin
            PrepareExit();
            exit;
          end
          else
            Continue;
        end;

        // Now HardLimit
        {$IFDEF USE_HARDLIM}
        if not Terminated and not BreakDesired then
        begin
          if HardLimit(filename2) then
            WriteLn(LNG_SUCCEED)
          else
            WriteLn(LNG_FAILED);
        end;
        {$ENDIF} // IFDEF USE_HARDLIM

        // Now MP3 Conv
        {$IFDEF USE_LAME}
        if not Terminated and not BreakDesired then
        begin
          if MP3Conv(filename2) then
            WriteLn(LNG_SUCCEED)
          else
            WriteLn(LNG_FAILED);
        end;
        {$ENDIF} // IFDEF USE_LAME
      end;
    end;
  finally
    fcol.Free;
  end;

  WriteLn(LNG_FINISHED);
  PrepareExit();
end.
